@@ -4,16 +4,16 @@ use Mojo::Asset::File;
 use Mojo::JSON;
 use Mojo::JSON::Pointer;
 use Mojo::URL;
-use Mojo::Util 'deprecated';
 use File::Basename ();
 use File::Spec;
 use Swagger2::SchemaValidator;
 
-our $VERSION = '0.75';
+our $VERSION = '0.76';
 
 # Should be considered internal
 our $SPEC_FILE = File::Spec->catfile(File::Basename::dirname(__FILE__), 'Swagger2', 'schema.json');
-our $JS_CLIENT = File::Spec->catfile(File::Basename::dirname(__FILE__), 'Swagger2', 'swagger2-client.js');
+our $JS_CLIENT
+  = File::Spec->catfile(File::Basename::dirname(__FILE__), 'Swagger2', 'swagger2-client.js');
 
 has api_spec => sub {
   my $self = shift;
@@ -35,16 +35,6 @@ has base_url => sub {
   return $url;
 };
 
-sub specification {
-  deprecated 'specification() will be removed.';
-  shift->_specification;
-}
-
-sub tree {
-  deprecated 'tree() is replaced by api_spec().';
-  shift->api_spec(@_);
-}
-
 has _specification => sub { shift->_validator->schema($SPEC_FILE)->schema };
 
 has _validator => sub {
@@ -56,9 +46,10 @@ sub ua  { shift->_validator->ua(@_) }
 sub url { shift->{url} }
 
 sub expand {
-  my $self  = shift;
-  my $class = Scalar::Util::blessed($self);
-  $class->new(%$self)->api_spec($self->_validator->schema($self->api_spec->data)->schema);
+  my $self   = shift;
+  my $class  = Scalar::Util::blessed($self);
+  my $schema = $self->_validator->schema($self->api_spec->data)->schema;
+  $class->new(%$self)->api_spec($schema);
 }
 
 sub javascript_client { Mojo::Asset::File->new(path => $JS_CLIENT) }
@@ -67,7 +58,7 @@ sub load {
   my $self = shift;
   delete $self->{base_url};
   $self->{url} = Mojo::URL->new(shift) if @_;
-  $self->{api_spec} = $self->_validator->_load_schema($self->{url});
+  $self->{api_spec} = $self->api_spec;
   $self;
 }
 
@@ -102,12 +93,8 @@ sub to_string {
   my $self = shift;
   my $format = shift || 'json';
 
-  if ($format eq 'yaml') {
-    return DumpYAML($self->api_spec->data);
-  }
-  else {
-    return Mojo::JSON::encode_json($self->api_spec->data);
-  }
+  return DumpYAML($self->api_spec->data) if $format eq 'yaml';
+  return Mojo::JSON::encode_json($self->api_spec->data);
 }
 
 sub validate {
@@ -131,7 +118,7 @@ Swagger2 - Swagger RESTful API Documentation
 
 =head1 VERSION
 
-0.75
+0.76
 
 =head1 DESCRIPTION
 
@@ -192,14 +179,6 @@ Holds a L<Mojo::JSON::Pointer> object containing your API specification.
 L<Mojo::URL> object that holds the location to the API endpoint.
 Note: This might also just be a dummy URL to L<http://example.com/>.
 
-=head2 specification
-
-DEPRECATED. If you need to change this, then you probably want L<JSON::Validator> instead.
-
-=head2 tree
-
-DEPRECATED. Use L</api_spec> instead.
-
 =head2 ua
 
   $ua = $self->ua;
